@@ -14,7 +14,6 @@ delimiter_p2 = re.compile(r'^ \*+')
 delimiter_patterns.append(delimiter_p1)
 delimiter_patterns.append(delimiter_p2)
 avail_patterns = []
-
 avail_patterns.append(re.compile(r'^ INITIAL POTENTIAL ENERGY'))
 avail_patterns.append(re.compile(r'^ ENSEMBLE TYPE'))
 
@@ -22,17 +21,24 @@ class Cp2kSystems(object):
     """
     deal with cp2k outputfile
     """
-    def __init__(self, log_file_name, xyz_file_name):
+    def __init__(self, log_file_name, xyz_file_name, restart=False):
         self.log_file_object = open(log_file_name, 'r')
         self.xyz_file_object = open(xyz_file_name, 'r')
         self.log_block_generator = self.get_log_block_generator()
         self.xyz_block_generator = self.get_xyz_block_generator()
+        self.restart_flag = restart
         self.cell=None
+
+        if self.restart_flag:
+            self.handle_single_log_frame(next(self.log_block_generator))
+
     def __del__(self):
         self.log_file_object.close()
         self.xyz_file_object.close()
+
     def __iter__(self):
         return self
+
     def __next__(self):
         info_dict = {}
         log_info_dict = self.handle_single_log_frame(next(self.log_block_generator))
@@ -40,10 +46,10 @@ class Cp2kSystems(object):
         eq1 = [v1==v2 for v1,v2 in zip(log_info_dict['atom_numbs'], xyz_info_dict['atom_numbs'])]
         eq2 = [v1==v2 for v1,v2 in zip(log_info_dict['atom_names'], xyz_info_dict['atom_names'])]
         eq3 = [v1==v2 for v1,v2 in zip(log_info_dict['atom_types'], xyz_info_dict['atom_types'])]
-        assert all(eq1), (log_info_dict,xyz_info_dict,'There may be errors in the file')
-        assert all(eq2), (log_info_dict,xyz_info_dict,'There may be errors in the file')
-        assert all(eq3), (log_info_dict,xyz_info_dict,'There may be errors in the file')
-        assert log_info_dict['energies']==xyz_info_dict['energies'], (log_info_dict['energies'],xyz_info_dict['energies'],'There may be errors in the file')
+        assert all(eq1), (log_info_dict,xyz_info_dict,'There may be errors in the file. If it is a restart task; use restart=True')
+        assert all(eq2), (log_info_dict,xyz_info_dict,'There may be errors in the file. If it is a restart task; use restart=True')
+        assert all(eq3), (log_info_dict,xyz_info_dict,'There may be errors in the file. If it is a restart task; use restart=True')
+        assert log_info_dict['energies']==xyz_info_dict['energies'], (log_info_dict['energies'], xyz_info_dict['energies'],'There may be errors in the file')
         info_dict.update(log_info_dict)
         info_dict.update(xyz_info_dict)
         return info_dict
@@ -111,6 +117,7 @@ class Cp2kSystems(object):
                     force_lines.append(line)
             if energy_pattern_1.match(line):
                 energy = float(energy_pattern_1.match(line).groupdict()['number']) * AU_TO_EV
+                #print('1to', energy)
             if energy_pattern_2.match(line):
                 energy = float(energy_pattern_2.match(line).groupdict()['number']) * AU_TO_EV
             if cell_length_pattern.match(line):
