@@ -12,11 +12,13 @@ import dpdata.deepmd.raw
 import dpdata.deepmd.comp
 import dpdata.qe.traj
 import dpdata.qe.scf
+import dpdata.abacus.scf
 import dpdata.siesta.output
 import dpdata.siesta.aiMD_output
 import dpdata.md.pbc
 import dpdata.gaussian.log
 import dpdata.amber.md
+import dpdata.amber.sqm
 import dpdata.cp2k.output
 from dpdata.cp2k.output import Cp2kSystems
 import dpdata.pwmat.movement
@@ -28,6 +30,7 @@ from monty.json import MSONable
 from monty.serialization import loadfn,dumpfn
 from dpdata.periodic_table import Element
 from dpdata.xyz.quip_gap_xyz import QuipGapxyzSystems
+from dpdata.amber.mask import pick_by_amber_mask, load_param_file
 
 
 class Register:
@@ -94,6 +97,8 @@ class System (MSONable) :
                 - ``deepmd/npy``: deepmd-kit compressed format (numpy binary)
                 - ``vasp/poscar``: vasp POSCAR
                 - ``qe/cp/traj``: Quantum Espresso CP trajectory files. should have: file_name+'.in' and file_name+'.pos'
+                - ``qe/pw/scf``: Quantum Espresso PW single point calculations. Both input and output files are required. If file_name is a string, it denotes the output file name. Input file name is obtained by replacing 'out' by 'in' from file_name. Or file_name is a list, with the first element being the input file name and the second element being the output filename.
+                - ``abacus/scf``: ABACUS plane wave scf. The directory containing INPUT file is required. 
                 - ``siesta/output``: siesta SCF output file
                 - ``siesta/aimd_output``: siesta aimd output file
                 - ``pwmat/atom.config``: pwmat atom.config
@@ -627,7 +632,7 @@ class System (MSONable) :
 
     @register_from_funcs.register_funcs("gro")
     @register_from_funcs.register_funcs("gromacs/gro")
-    def from_gromacs_gro(self, file_name) :
+    def from_gromacs_gro(self, file_name, format_atom_name=True) :
         """
         Load gromacs .gro file
 
@@ -636,7 +641,35 @@ class System (MSONable) :
         file_name : str
             The input file name
         """
-        self.data = dpdata.gromacs.gro.file_to_system_data(file_name)
+        self.data = dpdata.gromacs.gro.file_to_system_data(file_name, format_atom_name=format_atom_name)
+
+    @register_to_funcs.register_funcs("gromacs/gro")
+    def to_gromacs_gro(self, file_name=None, frame_idx=-1):
+        """
+        Dump the system in gromacs .gro format
+
+        Parameters
+        ----------
+        file_name : str or None
+            The output file name. If None, return the file content as a string
+        frame_idx : int
+            The index of the frame to dump
+        """
+        assert(frame_idx < len(self.data['coords']))
+        if frame_idx == -1:
+            strs = []
+            for idx in range(self.get_nframes()):
+                gro_str = dpdata.gromacs.gro.from_system_data(self.data, f_idx=idx)
+                strs.append(gro_str)
+            gro_str = "\n".join(strs)
+        else:
+            gro_str = dpdata.gromacs.gro.from_system_data(self.data, f_idx=frame_idx)
+        
+        if file_name is None:
+            return gro_str
+        else:
+            with open(file_name, 'w+') as fp:
+                fp.write(gro_str)
 
     @register_to_funcs.register_funcs("deepmd/npy")
     def to_deepmd_npy(self, folder, set_size = 5000, prec=np.float32) :
@@ -667,6 +700,13 @@ class System (MSONable) :
         Dump the system in deepmd raw format to `folder`
         """
         dpdata.deepmd.raw.dump(folder, self.data)
+    
+    @register_from_funcs.register_funcs('sqm/out')
+    def from_sqm_out(self, fname):
+        '''
+        Read from ambertools sqm.out
+        '''
+        self.data = dpdata.amber.sqm.to_system_data(fname)
 
     @register_from_funcs.register_funcs('siesta/output')
     def from_siesta_output(self, fname):
@@ -899,6 +939,10 @@ class System (MSONable) :
             return True
         return False
 
+    @nopbc.setter
+    def nopbc(self, value):
+        self.data['nopbc'] = value
+
     def shuffle(self):
         """Shuffle frames randomly."""
         idx = np.random.permutation(self.get_nframes())
@@ -920,7 +964,12 @@ class System (MSONable) :
         labeled_sys LabeledSystem
             The labeled system.
         """
-        import deepmd.DeepPot as DeepPot
+        try:
+            # DP 1.x
+            import deepmd.DeepPot as DeepPot
+        except ModuleNotFoundError:
+            # DP 2.x
+            from deepmd.infer import DeepPot
         if not isinstance(dp, DeepPot):
             dp = DeepPot(dp)
         type_map = dp.get_type_map()
@@ -945,6 +994,93 @@ class System (MSONable) :
             labeled_sys.append(this_sys)
         return labeled_sys
 
+    def pick_atom_idx(self, idx, nopbc=None):
+        """Pick atom index
+        
+        Parameters
+        ----------
+        idx: int or list or slice
+            atom index
+        nopbc: Boolen (default: None)
+            If nopbc is True or False, set nopbc
+
+        Returns
+        -------
+        new_sys: System
+            new system
+        """
+        new_sys = self.copy()
+        new_sys.data['coords'] = self.data['coords'][:, idx, :]
+        new_sys.data['atom_types'] = self.data['atom_types'][idx]
+        # recalculate atom_numbs according to atom_types
+        atom_numbs = np.bincount(new_sys.data['atom_types'], minlength=len(self.get_atom_names()))
+        new_sys.data['atom_numbs'] = list(atom_numbs)
+        if nopbc is True or nopbc is False:
+            new_sys.nopbc = nopbc
+        return new_sys
+
+    def remove_atom_names(self, atom_names):
+        """Remove atom names and all such atoms.
+        For example, you may not remove EP atoms in TIP4P/Ew water, which
+        is not a real atom. 
+        """
+        if isinstance(atom_names, str):
+            atom_names = [atom_names]
+        removed_atom_idx = []
+        for an in atom_names:
+            # get atom name idx
+            idx = self.data['atom_names'].index(an)
+            atom_idx = self.data['atom_types'] == idx
+            removed_atom_idx.append(atom_idx)
+        picked_atom_idx = ~np.any(removed_atom_idx, axis=0)
+        new_sys = self.pick_atom_idx(picked_atom_idx)
+        # let's remove atom_names
+        # firstly, rearrange atom_names and put these atom_names in the end
+        new_atom_names = list([xx for xx in new_sys.data['atom_names'] if xx not in atom_names])
+        new_sys.sort_atom_names(type_map=new_atom_names + atom_names)
+        # remove atom_names and atom_numbs
+        new_sys.data['atom_names'] = new_atom_names
+        new_sys.data['atom_numbs'] = new_sys.data['atom_numbs'][:len(new_atom_names)]
+        return new_sys
+
+    def pick_by_amber_mask(self, param, maskstr, pass_coords=False, nopbc=None):
+        """Pick atoms by amber mask
+        
+        Parameters
+        ----------
+        param: str or parmed.Structure
+          filename of Amber param file or parmed.Structure
+        maskstr: str
+          Amber masks
+        pass_coords: Boolen (default: False)
+            If pass_coords is true, the function will pass coordinates and 
+            return a MultiSystem. Otherwise, the result is
+            coordinate-independent, and the function will return System or
+            LabeledSystem.
+        nopbc: Boolen (default: None)
+            If nopbc is True or False, set nopbc
+        """
+        parm = load_param_file(param)
+        if pass_coords:
+            ms = MultiSystems()
+            for sub_s in self:
+                # TODO: this can computed in pararrel
+                idx = pick_by_amber_mask(parm, maskstr, sub_s['coords'][0])
+                ms.append(sub_s.pick_atom_idx(idx, nopbc=nopbc))
+            return ms
+        else:
+            idx = pick_by_amber_mask(parm, maskstr)
+            return self.pick_atom_idx(idx, nopbc=nopbc)
+
+    @register_from_funcs.register_funcs('amber/md')
+    def from_amber_md(self, file_name=None, parm7_file=None, nc_file=None, use_element_symbols=None):
+        # assume the prefix is the same if the spefic name is not given
+        if parm7_file is None:
+            parm7_file = file_name + ".parm7"
+        if nc_file is None:
+            nc_file = file_name + ".nc"
+        self.data = dpdata.amber.md.read_amber_traj(parm7_file=parm7_file, nc_file=nc_file, use_element_symbols=use_element_symbols, labeled=False)
+
 def get_cell_perturb_matrix(cell_pert_fraction):
     if cell_pert_fraction<0:
         raise RuntimeError('cell_pert_fraction can not be negative')
@@ -1026,7 +1162,7 @@ class LabeledSystem (System):
                 - ``gaussian/log``: gaussian logs
                 - ``gaussian/md``: gaussian ab initio molecular dynamics
                 - ``cp2k/output``: cp2k output file
-                - ``cp2k/aimd_output``: cp2k aimd output  dir(contains *pos*.xyz and *.log file)
+                - ``cp2k/aimd_output``: cp2k aimd output  dir(contains *pos*.xyz and *.log file); optional `restart=True` if it is a cp2k restarted task.
                 - ``pwmat/movement``: pwmat md output file
                 - ``pwmat/out.mlmd``: pwmat scf output file
 
@@ -1091,13 +1227,21 @@ class LabeledSystem (System):
         return ('virials' in self.data)
 
     @register_from_funcs.register_funcs('cp2k/aimd_output')
-    def from_cp2k_aimd_output(self, file_dir):
+    def from_cp2k_aimd_output(self, file_dir, restart=False):
         xyz_file=sorted(glob.glob("{}/*pos*.xyz".format(file_dir)))[0]
         log_file=sorted(glob.glob("{}/*.log".format(file_dir)))[0]
-        for info_dict in Cp2kSystems(log_file, xyz_file):
+        for info_dict in Cp2kSystems(log_file, xyz_file, restart):
             l = LabeledSystem(data=info_dict)
             self.append(l)
 
+    # @register_from_funcs.register_funcs('cp2k/restart_aimd_output')
+    # def from_cp2k_aimd_output(self, file_dir, restart=True):
+    #     xyz_file = sorted(glob.glob("{}/*pos*.xyz".format(file_dir)))[0]
+    #     log_file = sorted(glob.glob("{}/*.log".format(file_dir)))[0]
+    #     for info_dict in Cp2kSystems(log_file, xyz_file, restart):
+    #         l = LabeledSystem(data=info_dict)
+    #         self.append(l)
+
     @register_from_funcs.register_funcs('fhi_aims/md')
     def from_fhi_aims_output(self, file_name, md=True, begin=0, step =1):
         self.data['atom_names'], \
@@ -1230,6 +1374,11 @@ class LabeledSystem (System):
             self.data['virials'], \
             = dpdata.qe.scf.get_frame(file_name)
         self.rot_lower_triangular()
+    
+    @register_from_funcs.register_funcs('abacus/scf')
+    def from_abacus_pw_scf(self, file_name) :
+        self.data = dpdata.abacus.scf.get_frame(file_name)
+        self.rot_lower_triangular()
 
     @register_from_funcs.register_funcs('siesta/output')
     def from_siesta_output(self, file_name) :
@@ -1269,7 +1418,7 @@ class LabeledSystem (System):
         self.from_gaussian_log(file_name, md=True)
 
     @register_from_funcs.register_funcs('amber/md')
-    def from_amber_md(self, file_name=None, parm7_file=None, nc_file=None, mdfrc_file=None, mden_file=None):
+    def from_amber_md(self, file_name=None, parm7_file=None, nc_file=None, mdfrc_file=None, mden_file=None, mdout_file=None, use_element_symbols=None):
         # assume the prefix is the same if the spefic name is not given
         if parm7_file is None:
             parm7_file = file_name + ".parm7"
@@ -1279,7 +1428,9 @@ class LabeledSystem (System):
             mdfrc_file = file_name + ".mdfrc"
         if mden_file is None:
             mden_file = file_name + ".mden"
-        self.data = dpdata.amber.md.read_amber_traj(parm7_file, nc_file, mdfrc_file, mden_file)
+        if mdout_file is None:
+            mdout_file = file_name + ".mdout"
+        self.data = dpdata.amber.md.read_amber_traj(parm7_file, nc_file, mdfrc_file, mden_file, mdout_file, use_element_symbols)
 
     @register_from_funcs.register_funcs('cp2k/output')
     def from_cp2k_output(self, file_name) :
@@ -1439,6 +1590,53 @@ class LabeledSystem (System):
             entries.append(entry)
         return entries
 
+    def correction(self, hl_sys):
+        """Get energy and force correction between self and a high-level LabeledSystem.
+        The self's coordinates will be kept, but energy and forces will be replaced by
+        the correction between these two systems.
+
+        Note: The function will not check whether coordinates and elements of two systems
+        are the same. The user should make sure by itself.
+
+        Parameters
+        ----------
+        hl_sys: LabeledSystem
+            high-level LabeledSystem
+        Returns
+        ----------
+        corrected_sys: LabeledSystem
+            Corrected LabeledSystem
+        """
+        if not isinstance(hl_sys, LabeledSystem):
+            raise RuntimeError("high_sys should be LabeledSystem")
+        corrected_sys = self.copy()
+        corrected_sys.data['energies'] = hl_sys.data['energies'] - self.data['energies']
+        corrected_sys.data['forces'] = hl_sys.data['forces'] - self.data['forces']
+        if 'virials' in self.data and 'virials' in hl_sys.data:
+            corrected_sys.data['virials'] = hl_sys.data['virials'] - self.data['virials']
+        return corrected_sys
+
+    def pick_atom_idx(self, idx, nopbc=None):
+        """Pick atom index
+        
+        Parameters
+        ----------
+        idx: int or list or slice
+            atom index
+        nopbc: Boolen (default: None)
+            If nopbc is True or False, set nopbc
+
+        Returns
+        -------
+        new_sys: LabeledSystem
+            new system
+        """
+        new_sys = System.pick_atom_idx(self, idx, nopbc=nopbc)
+        # forces
+        new_sys.data['forces'] = self.data['forces'][:, idx, :]
+        return new_sys
+
+
 class MultiSystems:
     '''A set containing several systems.'''
 
@@ -1607,13 +1805,38 @@ class MultiSystems:
         return self
 
     def predict(self, dp):
-        import deepmd.DeepPot as DeepPot
+        try:
+            # DP 1.x
+            import deepmd.DeepPot as DeepPot
+        except ModuleNotFoundError:
+            # DP 2.x
+            from deepmd.infer import DeepPot
         if not isinstance(dp, DeepPot):
             dp = DeepPot(dp)
         new_multisystems = dpdata.MultiSystems()
         for ss in self:
             new_multisystems.append(ss.predict(dp))
         return new_multisystems
+    
+    def pick_atom_idx(self, idx, nopbc=None):
+        """Pick atom index
+        
+        Parameters
+        ----------
+        idx: int or list or slice
+            atom index
+        nopbc: Boolen (default: None)
+            If nopbc is True or False, set nopbc
+
+        Returns
+        -------
+        new_sys: MultiSystems
+            new system
+        """
+        new_sys = MultiSystems()
+        for ss in self:
+            new_sys.append(ss.pick_atom_idx(idx, nopbc=nopbc))
+        return new_sys
 
 
 def check_System(data):
@@ -1652,7 +1875,4 @@ def elements_index_map(elements,standard=False,inverse=False):
         return dict(zip(range(len(elements)),elements))
     else:
         return dict(zip(elements,range(len(elements))))
-
-
-
 # %%
